-- vertex

#version 460 core

layout(location = 0) in vec4 pos;

uniform vec2	scale;
uniform mat4	proj;

out vec2 tex;
out vec2 viewRay;

void main(void)
{
	tex         = 0.5 * ( pos.xy + vec2 ( 1.0 ) );
	//viewRay.x   = -pos.x / proj [0][0];
	//viewRay.y   = -pos.y / proj [1][1];	
	gl_Position = vec4 ( pos.xy, 0.0, 1.0 );
}

-- fragment

#version 460 core

uniform sampler2D   lastMip;
uniform int         lastMipWidth;
uniform int         lastMipHeight;

in vec2 tex;

void main(void)
{
	vec4	texels;
	vec3	extra;

	texels.x = texture       ( lastMip, tex ).x;
	texels.y = textureOffset ( lastMip, tex, ivec2(-1, 0) ).x;
	texels.z = textureOffset ( lastMip, tex, ivec2(-1,-1) ).x;
	texels.w = textureOffset ( lastMip, tex, ivec2( 0,-1) ).x;
	
	float	maxZ = max ( max ( texels.x, texels.y ), max ( texels.z, texels.w ) );

		// if we are reducing an odd-width texture then the edge fragments have to fetch additional texels
	if ( ( (lastMipWidth & 1) != 0 ) && ( int(gl_FragCoord.x) == lastMipWidth - 3 ) ) 
	{
			// if both edges are odd, fetch the top-left corner texel
		if ( ( (lastMipHeight & 1) != 0 ) && ( int(gl_FragCoord.y) == lastMipHeight - 3 ) ) 
		{
			extra.z = textureOffset ( lastMip, tex, ivec2( 1, 1) ).x;
			maxZ    = max           ( maxZ, extra.z );
		}

		extra.x = textureOffset ( lastMip, tex, ivec2( 1, 0) ).x;
		extra.y = textureOffset ( lastMip, tex, ivec2( 1,-1) ).x;
		maxZ    = max           ( maxZ, max( extra.x, extra.y ) );
	} 
	else	// if we are reducing an odd-height texture then the edge fragments have to fetch additional texels
	if ( ( (lastMipHeight & 1) != 0 ) && ( int(gl_FragCoord.y) == lastMipHeight - 3 ) ) 
	{
		extra.x = textureOffset ( lastMip, tex, ivec2( 0, 1) ).x;
		extra.y = textureOffset ( lastMip, tex, ivec2(-1, 1) ).x;
		maxZ    = max           ( maxZ, max( extra.x, extra.y ) );
	}
	
	gl_FragDepth = maxZ;
}
